#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <libgen.h>
#include <signal.h>
#include <limits.h>

#include "mjpg_streamer.h"

#ifdef MX28_1024
#define MOTION_FILE_PATH    "../../../Data/motion_1024.bin"
#else
#define MOTION_FILE_PATH    "../../../Data/motion_4096.bin"
#endif


#include "LinuxDARwIn.h"

#define INI_FILE_PATH       "../../../Data/config.ini"

using namespace Robot;
void error(const char *msg)
{
    perror(msg);
    exit(0);
}

int main(int argc, char *argv[])
{

	int prevvalue1, prevvalue2, prevvalue3;
	int value1, value2, value3;
		printf( "\n===== Master/Slave: Server for DARwIn =====\n\n");

	
	  
	
	//////////////////// Framework Initialize ////////////////////////////	
	LinuxCM730 linux_cm730("/dev/ttyUSB0");
	CM730 cm730(&linux_cm730);
	if(cm730.Connect() == false)
	{
		printf("Fail to connect CM-730!\n");
		return 0;
	}
	/////////////////////////////////////////////////////////////////////
	minIni* ini = new minIni(INI_FILE_PATH);
    	if(MotionManager::GetInstance()->Initialize(&cm730) == false)
   	{
   	     printf("Initializing Motion Manager failed!\n");
   	     return 0;
	}
	/////////
	MotionManager::GetInstance()->AddModule((MotionModule*)Action::GetInstance());
    LinuxMotionTimer *motion_timer = new LinuxMotionTimer(MotionManager::GetInstance());
    motion_timer->Start();
	int firm_ver = 0;
    if(cm730.ReadByte(JointData::ID_HEAD_PAN, MX28::P_VERSION, &firm_ver, 0)  != CM730::SUCCESS)
    {
        fprintf(stderr, "Can't read firmware version from Dynamixel ID %d!! \n\n", JointData::ID_HEAD_PAN);
        exit(0);
    }

    if(0 < firm_ver && firm_ver < 27)
    {
#ifdef MX28_1024
        Action::GetInstance()->LoadFile(MOTION_FILE_PATH);
#else
        fprintf(stderr, "MX-28's firmware is not support 4096 resolution!! \n");
        fprintf(stderr, "Upgrade MX-28's firmware to version 27(0x1B) or higher.\n\n");
        exit(0);
#endif
    }
    else if(27 <= firm_ver)
    {
#ifdef MX28_1024
        fprintf(stderr, "MX-28's firmware is not support 1024 resolution!! \n");
        fprintf(stderr, "Remove '#define MX28_1024' from 'MX28.h' file and rebuild.\n\n");
        exit(0);
#else
        Action::GetInstance()->LoadFile((char*)MOTION_FILE_PATH);
#endif
    }
    else
        exit(0);
	/////////
	MotionManager::GetInstance()->LoadINISettings(ini);
	
	Action::GetInstance()->m_Joint.SetEnableBody(true, true); // turn on the torque to play the action
        MotionManager::GetInstance()->SetEnable(true);

	Action::GetInstance()->Start(9); // play walk ready position

		sleep(5);

	Action::GetInstance()->m_Joint.SetEnableBody(false, false); // disables the torque
        MotionManager::GetInstance()->SetEnable(false);

    cm730.WriteWord(JointData::ID_R_SHOULDER_PITCH, MX28::P_TORQUE_ENABLE, 0, 0);
    cm730.WriteWord(JointData::ID_R_SHOULDER_ROLL,  MX28::P_TORQUE_ENABLE, 0, 0);
    cm730.WriteWord(JointData::ID_R_ELBOW,          MX28::P_TORQUE_ENABLE, 0, 0);

    cm730.WriteByte(JointData::ID_L_SHOULDER_PITCH, MX28::P_P_GAIN, 8, 0);
    cm730.WriteByte(JointData::ID_L_SHOULDER_ROLL,  MX28::P_P_GAIN, 8, 0);
    cm730.WriteByte(JointData::ID_L_ELBOW,          MX28::P_P_GAIN, 8, 0);

	




	






    int sockfd, portno, n, value;
    struct sockaddr_in serv_addr;
    struct hostent *server;

    char buffer[256];
	int disp[3] = {1};
    if (argc < 3) {
       fprintf(stderr,"usage %s hostname port\n", argv[0]);
       exit(0);
    }
    portno = atoi(argv[2]);
    sockfd = socket(AF_INET, SOCK_STREAM, 0);
    if (sockfd < 0)
        error("ERROR opening socket");
    server = gethostbyname(argv[1]);
    if (server == NULL) {
        fprintf(stderr,"ERROR, no such host\n");
        exit(0);
    }
    bzero((char *) &serv_addr, sizeof(serv_addr));
    serv_addr.sin_family = AF_INET;
    bcopy((char *)server->h_addr,
         (char *)&serv_addr.sin_addr.s_addr,
         server->h_length);
    serv_addr.sin_port = htons(portno);
    if (connect(sockfd,(struct sockaddr *) &serv_addr,sizeof(serv_addr)) < 0)
        error("ERROR connecting");
    //////


	while(1){
	Action::GetInstance()->m_Joint.SetEnableBody(false, false); // torque off
        MotionManager::GetInstance()->SetEnable(false);
    printf("Please enter the message: ");
    bzero(buffer,256);
 	

		if(cm730.ReadWord(JointData::ID_R_SHOULDER_PITCH, MX28::P_PRESENT_POSITION_L, &value1, 0) == CM730::SUCCESS) // Read the shoulder pitch position
		{

			//disp[0] =  value1 - prevvalue1;
			//printf("disp: %4d",disp[0]);
			//prevvalue1 = value1;
			disp[0] = value1;
		}

		if(cm730.ReadWord(JointData::ID_R_SHOULDER_ROLL, MX28::P_PRESENT_POSITION_L, &value2, 0) == CM730::SUCCESS)// Read the shoulder roll position
		{

			//disp[1] = value2 - prevvalue2;
			//printf("disp: %4d",disp[1]);
			//prevvalue2 = value2;
			disp[1]= value2;
			
		}


		if(cm730.ReadWord(JointData::ID_R_ELBOW, MX28::P_PRESENT_POSITION_L, &value3, 0) == CM730::SUCCESS)// Read the shoulder 
		{
		
			//disp[2] = value3-prevvalue3;
			//printf("disp: %4d",disp[2]);
			//prevvalue3 = value3;
			disp[2] = value3;
		}		

    n = write(sockfd,disp,sizeof(disp)); // sending the dynamixel position as an array [disp] to the server 
    if (n < 0)
         error("ERROR writing to socket");
    bzero(buffer,256);

    n = read(sockfd,buffer,1);
    if (n < 0)
         error("ERROR reading from socket");
    printf("%s\n",buffer);

	}
    close(sockfd);
    return 0;
}
